"""pandas utilities."""

import pandas as pd


def set_pandas_display_options() -> None:
    """Set pandas display options."""
    # Ref: https://stackoverflow.com/a/52432757/
    display = pd.options.display

    display.max_columns = 1000
    display.max_rows = 1000
    display.max_colwidth = 199
    display.width = None
    # display.precision = 2  # set as needed



"""mgzip utilities."""
import logging
import math
import multiprocessing

import mgzip

from wpdatautil.humanfriendly import format_size
from wpdatautil.timeit import Timer

_COMPRESS_LEVEL = 6
_INVERSE_PI = 1 / math.pi
_CPU_COUNT_SCALER = (2 ** 30) ** _INVERSE_PI

log = logging.getLogger(__name__)


def _num_threads(num_bytes: int) -> int:
    """Return the number of threads to use for compression or decompression."""
    max_usable_cpu_count = multiprocessing.cpu_count() * 0.95
    return max(1, math.floor(min(_CPU_COUNT_SCALER, num_bytes ** _INVERSE_PI) * max_usable_cpu_count / _CPU_COUNT_SCALER))


def compress(uncompressed: bytes, description: str = "blob") -> bytes:
    """Compress the given bytes in parallel using `mgzip`.

    The number of compression threads is determined automatically based on the input size and the CPU count.
    """
    len_uncompressed = len(uncompressed)
    len_uncompressed_formatted = format_size(len_uncompressed)
    num_threads = _num_threads(len_uncompressed)
    log.debug(f"Compressing {description} from {len_uncompressed_formatted} using {num_threads} threads.")
    timer = Timer()
    compressed = mgzip.compress(uncompressed, compresslevel=_COMPRESS_LEVEL, thread=num_threads)
    log.debug(f"Compressed {description} to {format_size(len(compressed))} from {len_uncompressed_formatted} in {timer} using {num_threads} threads.")
    return compressed


def decompress(compressed: bytes) -> bytes:
    """Decompress the given bytes in parallel using `mgzip`.

    The number of decompression threads is determined automatically based on the input size and the CPU count.
    """
    return mgzip.decompress(compressed, thread=_num_threads(len(compressed)))



"""logging psutil utilities."""
import logging

from psutil import cpu_percent, virtual_memory

# Reference for using a logging filter: https://stackoverflow.com/a/61830838/


class PsutilFilter(logging.Filter):
    """psutil logging filter."""

    def filter(self, record: logging.LogRecord) -> bool:
        """Add contextual information about the currently used CPU and virtual memory percentages into the `psutil` attribute of the given log record.

        An example of the set value is "c18m68", which means that 18% of the system-wide CPU and 68% of the physical memory are in use.
        """
        record.psutil = f"c{cpu_percent():02.0f}m{virtual_memory().percent:02.0f}"  # type: ignore
        return True  # True means don't discard the record.
